/*
  Copyright (c) 2014 by I.Kato @ BS21 Lab
  SRAM512K Shield (BS21LAB-018) library for Arduino
  
  Based:
    Copyright (c) 2010 by arms22 (arms22 at gmail.com)
    Microchip 23x256 SPI SRAM library for Arduino
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#include "SPISRAM512K.h"
#include <avr/delay.h>


// SRAM 512K Shield (BS21LAB-018) 用ライブラリ
// ※ 本ライブラリは、arms22氏作成の SPISRAM ライブラリをベースにしています。

// 制御IC である ATmega8515L-8PU が 8MHz 駆動と遅い為、Arduino 側が 16MHz 駆動の時は、
// Arduino 側の SPI 転送速度を SPI_CLOCK_DIV8 にまで遅くする必要が有ります。
// さらに、ATmega8515L-8PU が内部処理する為の待ち時間として、SPI.transfer() する度に
// 5μ秒ほど待ってやる必要が有ります。この結果、読み書き速度は、余り早いとは言えません。

// SRAM の容量が 512kバイトの為、指定できるアドレスは 19bit長になります。


// INSTRUCTION SET
#define READ	0x80			// Read data from memory
#define WRITE	0x00			// Write data to memory


// コンストラクタ
// CSピンを指定する必要が有ります
SPISRAM512K::SPISRAM512K (byte ncsPin)
	: _ncsPin(ncsPin)
{
}

// 初期化
// SPI初期化と CSピンを出力モードにします
void SPISRAM512K::init (void)
{
	SPI.begin();

	// SPI の設定を SRAM512Kシールド用に設定します
	setSpiConditionForSRAM512K();

	// Set CS to high so a connected chip will be "deselected" by default
	digitalWrite (_ncsPin, HIGH);

	// When the CS pin is set as OUTPUT, it can be used as
	// a general purpose output port (it doesn't influence
	// SPI operations).
	pinMode (_ncsPin, OUTPUT);

}


// SPI の設定を SRAM512Kシールド用に設定します
// SPI 通信速度を 2MHz に設定
void SPISRAM512K::setSpiConditionForSRAM512K (void)
{
#ifndef __arm__

// Arduino Uno R3 等の時

	SPI.setBitOrder (MSBFIRST);
#if F_CPU <= 8000000UL
	SPI.setClockDivider (SPI_CLOCK_DIV4);  // Arduino が 8MHz の時
#else
	SPI.setClockDivider (SPI_CLOCK_DIV8);  // Arduino が 16MHz の時
#endif
	SPI.setDataMode (SPI_MODE0);

#else // __arm__

// Arduino Due の時

	SPI.setClockDivider (_ncsPin, 42);  // 84MHz / 42 = 2MHz
	SPI.setDataMode     (_ncsPin, SPI_MODE0);
	SPI.setBitOrder     (_ncsPin, MSBFIRST);

#endif // __arm__
}


// SPI の設定を Default 設定にします
// SPI 通信速度を 4MHz に設定
// ※ Default 設定に戻しただけでは駄目な SPI 機器があるかもしれません。
//    そのような SPI 機器は、ドキュメント等を見て、正しい SPI 設定にするようにしてください。
void SPISRAM512K::setSpiConditionForDefault (void)
{
#ifndef __arm__

// Arduino Uno R3 等の時

	SPI.setBitOrder     (MSBFIRST);
	SPI.setClockDivider (SPI_CLOCK_DIV4);
	SPI.setDataMode     (SPI_MODE0);

#else // __arm__

// Arduino Due の時

	SPI.setClockDivider (_ncsPin, 21);  // 84MHz / 21 = 4MHz
	SPI.setDataMode     (_ncsPin, SPI_MODE0);
	SPI.setBitOrder     (_ncsPin, MSBFIRST);

#endif // __arm__
}


// １バイト読み出し
byte SPISRAM512K::read (unsigned long address)
{
	_buf[0] = READ + ((address >> 16) & 0x0F);
	_buf[1] = (address >> 8) & 0xFF;
	_buf[2] = address & 0xFF;
	_buf[3] = 0x00;  // dummy data
	return transfer(4);
}

// 複数バイト読み出し
void SPISRAM512K::read (unsigned long address, byte *buffer, unsigned int size)
{
	select();
	_delay_us(5); // 5 micro seconds delay
	SPI.transfer (READ + ((address >> 16) & 0x0F));
	_delay_us(5); // 5 micro seconds delay
	SPI.transfer ((address >> 8) & 0xFF);
	_delay_us(5); // 5 micro seconds delay
	SPI.transfer (address & 0xFF);
	for (unsigned int i = 0; i < size; i++) {
		_delay_us(5); // 5 micro seconds delay
		*buffer++ = SPI.transfer (0);
	}
	_delay_us(5); // 5 micro seconds delay
	deselect();
}

// １バイト書き込み
void SPISRAM512K::write (unsigned long address, byte data)
{
	_buf[0] = WRITE + ((address >> 16) & 0x0F);
	_buf[1] = (address >> 8) & 0xFF;
	_buf[2] = address & 0xFF;
	_buf[3] = data;
	transfer(4);
}

// 複数バイト書き込み
void SPISRAM512K::write (unsigned long address, byte *buffer, unsigned int size)
{
	select();
	_delay_us(5); // 5 micro seconds delay
	SPI.transfer(WRITE + ((address >> 16) & 0x0F));
	_delay_us(5); // 5 micro seconds delay
	SPI.transfer((address >> 8) & 0xFF);
	_delay_us(5); // 5 micro seconds delay
	SPI.transfer(address & 0xFF);
	for (unsigned int i = 0; i < size; i++) {
		_delay_us(5); // 5 micro seconds delay
		SPI.transfer(*buffer++);
	}
	_delay_us(5); // 5 micro seconds delay
	deselect();
}

// CSピンを選択
void SPISRAM512K::select (void)
{
	digitalWrite (_ncsPin, LOW);
}

// CSピンを解放
void SPISRAM512K::deselect (void)
{
	digitalWrite (_ncsPin, HIGH);
}

// SPI通信
byte SPISRAM512K::transfer (byte length)
{
	byte ret, i;
	select();
	for (i = 0; i < length; i++) {
		_delay_us(5); // 5 micro seconds delay
		ret = SPI.transfer(_buf[i]);
	}
	_delay_us(5); // 5 micro seconds delay
	deselect();
	return ret;
}

